/* VARIABLES */
$border-color-nofocus: $primary;
$border-title-color-nofocus: $primary-lighten-3;
$border-color-focus: $accent;
$border-title-color-focus: $accent;
/* Main Screen */
#sql_client {
    layout: grid;
    grid-size: 2 2;
    grid-columns: 1fr 3fr;
}
/* ALL WIDGETS */
SchemaViewer, CodeEditor, ResultsViewer {
    height: 100%;
    border: round $border-color-nofocus;
    border-title-color: $border-title-color-nofocus;
    background: $boost;
}
SchemaViewer:focus, CodeEditor:focus-within, ResultsViewer:focus-within {
    border: round $border-color-focus;
    border-title-color: $border-title-color-focus;
}

SchemaViewer, TextContainer, ResultsTable {
    scrollbar-background: $surface;
    scrollbar-background-hover: $surface;
    scrollbar-background-active: $surface;
    scrollbar-corner-color: $surface;
    
    scrollbar-color: $panel-lighten-1;
    scrollbar-color-hover: $primary;
    scrollbar-color-active: $accent;
}

/* QUERY EDITOR */

/* SCHEMA VIEWER */
SchemaViewer {
    padding: 0 1;
    row-span: 2;
}
SchemaViewer .tree--guides {
    color: $primary;
}
SchemaViewer .tree--guides-hover {
    color: $accent;
}
SchemaViewer .tree--guides-selected {
    color: $secondary;
}

/* RESULTS VIEWER */

ResultsViewer.non-responsive {
    border: round $panel-lighten-1;
}

ResultsTable.datatable--cursor .datatable--header-cursor {
    background: $boost;
}
ResultsViewer .datatable--hover {
    background: $boost;
}
ResultsViewer .datatable--header {
    background: $boost;
}
ResultsViewer .datatable--header-hover {
    background: $primary-background;
}

/* ERROR MODAL */
ErrorModal {
    align: center middle;
    padding: 0;
}
#outer {
    border: round $error;
    background: $surface;
    margin: 5 10;
    padding: 1 2;
    max-width: 88;
}

#error_header {
    dock: top;
    color: $text-muted;
    margin: 0 0 1 0;
    padding: 0 1;
}

#inner {
    border: round $panel-lighten-1;
    padding: 1 1 1 2;
}

#error_info {
    padding: 0 3 0 0;
}

#error_footer {
    dock: bottom;
    color: $text-muted;
    margin: 1 0 0 0;
    padding: 0 1;
}

/* FILE MODAL */

FilenameModal {
    align: center middle;
    padding: 0;
}
#file_container {
    border: round $primary;
    background: $surface;
    margin: 5 10;
    padding: 1 2;
    height: 11;
    min-width: 40;
    max-width: 88;
}

#file_header {
    dock: top;
    color: $text-muted;
    margin: 0 0 1 0;
    padding: 0 1;
}

/* #inner {
    border: round $panel-lighten-1;
    padding: 1 1 1 2;
}

#error_info {
    padding: 0 3 0 0;
} */

#file_footer {
    dock: bottom;
    color: $text-muted;
    margin: 1 0 0 0;
    padding: 0 1;
}